package sun.security.jgss.krb5;

import sun.security.action.GetPropertyAction;
import sun.security.jgss.GSSToken;
import sun.security.krb5.internal.AuthorizationData;
import java.net.InetAddress;
import com.sun.security.jgss.AuthorizationDataEntry;
import sun.security.util.DerValue;
import java.io.InputStream;
import org.ietf.jgss.GSSException;
import java.io.IOException;
import sun.security.krb5.KrbException;
import sun.security.krb5.EncryptionKey;
import sun.security.krb5.Checksum;
import sun.security.krb5.internal.KerberosTime;
import sun.security.krb5.Credentials;
import sun.security.krb5.KrbApReq;

class InitSecContextToken extends InitialToken
{
    private static final boolean ACCEPTOR_USE_INITIATOR_SEQNUM;
    private KrbApReq apReq;
    
    InitSecContextToken(Krb5Context krb5Context, Credentials credentials, Credentials tgsCred) throws KrbException, IOException, GSSException {
        this.apReq = null;
        int mutualAuthState = krb5Context.getMutualAuthState();
        boolean useSubKey = true;
        boolean useSeqNumber = true;
        Checksum checksum = new InitialToken.OverloadedChecksum(krb5Context, credentials, tgsCred).getChecksum();
        krb5Context.setTktFlags(tgsCred.getFlags());
        krb5Context.setAuthTime(new KerberosTime(tgsCred.getAuthTime()).toString());
        this.apReq = new KrbApReq(tgsCred, (boolean)mutualAuthState, (boolean)useSubKey, (boolean)useSeqNumber, checksum);
        krb5Context.resetMySequenceNumber((int)this.apReq.getSeqNumber());
        EncryptionKey subKey = this.apReq.getSubKey();
        if (subKey != null) {
            krb5Context.setKey(1, subKey);
        }
        else {
            krb5Context.setKey(0, tgsCred.getSessionKey());
        }
        if (mutualAuthState == 0) {
            krb5Context.resetPeerSequenceNumber(InitSecContextToken.ACCEPTOR_USE_INITIATOR_SEQNUM ? ((int)this.apReq.getSeqNumber()) : 0);
        }
    }
    
    InitSecContextToken(Krb5Context contextFlags, Krb5AcceptCredential cred, InputStream in) throws IOException, GSSException, KrbException {
        this.apReq = null;
        if ((in.read() << 8 | in.read()) != 0x100) {
            throw new GSSException(10, -1, "AP_REQ token id does not match!");
        }
        byte[] byteArray = new DerValue(in).toByteArray();
        InetAddress initiatorAddress = null;
        if (contextFlags.getChannelBinding() != null) {
            initiatorAddress = contextFlags.getChannelBinding().getInitiatorAddress();
        }
        this.apReq = new KrbApReq(byteArray, cred, initiatorAddress);
        EncryptionKey sessionKey = this.apReq.getCreds().getSessionKey();
        EncryptionKey subKey = this.apReq.getSubKey();
        if (subKey != null) {
            contextFlags.setKey(1, subKey);
        }
        else {
            contextFlags.setKey(0, sessionKey);
        }
        InitialToken.OverloadedChecksum o = new InitialToken.OverloadedChecksum(contextFlags, this.apReq.getChecksum(), sessionKey, subKey);
        o.setContextFlags(contextFlags);
        Credentials delegatedCreds = o.getDelegatedCreds();
        if (delegatedCreds != null) {
            contextFlags.setDelegCred(Krb5InitCredential.getInstance((Krb5NameElement)contextFlags.getSrcName(), delegatedCreds));
        }
        Integer seqNumber = this.apReq.getSeqNumber();
        int n = (seqNumber != null) ? ((int)seqNumber) : 0;
        contextFlags.resetPeerSequenceNumber(n);
        if (!contextFlags.getMutualAuthState()) {
            contextFlags.resetMySequenceNumber(InitSecContextToken.ACCEPTOR_USE_INITIATOR_SEQNUM ? n : false);
        }
        contextFlags.setAuthTime(new KerberosTime(this.apReq.getCreds().getAuthTime()).toString());
        contextFlags.setTktFlags(this.apReq.getCreds().getFlags());
        AuthorizationData authzData = this.apReq.getCreds().getAuthzData();
        if (authzData == null) {
            contextFlags.setAuthzData((AuthorizationDataEntry[])null);
        }
        else {
            AuthorizationDataEntry[] authzData2 = new AuthorizationDataEntry[authzData.count()];
            for (int i = 0; i < authzData.count(); ++i) {
                authzData2[i] = new AuthorizationDataEntry(authzData.item(i).adType, authzData.item(i).adData);
            }
            contextFlags.setAuthzData(authzData2);
        }
        return;
    }
    
    public final KrbApReq getKrbApReq() {
        return this.apReq;
    }
    
    @Override
    public final byte[] encode() throws IOException {
        byte[] message = this.apReq.getMessage();
        byte[] dest = new byte[2 + message.length];
        GSSToken.writeInt(256, dest, 0);
        System.arraycopy(message, 0, dest, 2, message.length);
        return dest;
    }
    
    static {
        String s = "sun.security.krb5.acceptor.sequence.number.nonmutual";
        String privilegedGetProperty = GetPropertyAction.privilegedGetProperty(s, "initiator");
        if (privilegedGetProperty.equals("initiator")) {
            ACCEPTOR_USE_INITIATOR_SEQNUM = true;
        }
        else if (privilegedGetProperty.equals("zero") || privilegedGetProperty.equals("0")) {
            ACCEPTOR_USE_INITIATOR_SEQNUM = false;
        }
        else {
            throw new AssertionError("Unrecognized value for " + s + ": " + privilegedGetProperty);
        }
    }
}
